@@ -15,7 +15,7 @@ OSErr eventHandler(const AppleEvent *, AppleEvent *, SRefCon )
 		if (mainwin) break;
 	}
 	if (mainwin) {
-		mainwin->actionReloadCompile();
+		mainwin->actionReloadRenderCSG();
 	}
 	return noErr;
 }
@@ -24,7 +24,7 @@ void installAppleEventHandlers()
 {
 	// Reload handler
   OSErr err = AEInstallEventHandler('SCAD', 'relo', NewAEEventHandlerUPP(eventHandler), 0, true);
-  require_noerr(err, CantInstallAppleEventHandler);
+  __Require_noErr(err, CantInstallAppleEventHandler);
 	return;
 
 CantInstallAppleEventHandler:
