@@ -61,6 +61,11 @@ void CGALEvaluator::process(CGAL_Nef_polyhedron &target, const CGAL_Nef_polyhedr
  	if (target.dim != 2 && target.dim != 3) {
  		assert(false && "Dimension of Nef polyhedron must be 2 or 3");
  	}
+    // Intersecting something with nothing results in nothing
+	if (src.isEmpty() && op == CGE_INTERSECTION) {
+        target = src;
+        return;
+    }
 	if (src.isEmpty()) return; // Empty polyhedron. This can happen for e.g. square([0,0])
 	if (target.isEmpty() && op != CGE_UNION) return; // empty op <something> => empty
 	if (target.dim != src.dim) return; // If someone tries to e.g. union 2d and 3d objects
@@ -126,7 +131,7 @@ CGAL_Nef_polyhedron CGALEvaluator::applyHull(const CgaladvNode &node)
 	CGAL_Nef_polyhedron N;
 	std::list<CGAL_Nef_polyhedron2*> polys;
 	std::list<CGAL_Nef_polyhedron2::Point> points2d;
-	std::list<CGAL_Polyhedron::Vertex::Point_3> points3d;
+	std::vector<CGAL_Polyhedron::Vertex::Point_3> points3d;
 	int dim = 0;
 	BOOST_FOREACH(const ChildItem &item, this->visitedchildren[node.index()]) {
 		const AbstractNode *chnode = item.first;
@@ -141,6 +146,9 @@ CGAL_Nef_polyhedron CGALEvaluator::applyHull(const CgaladvNode &node)
 			PRINT("WARNING: hull() does not support mixing 2D and 3D objects.");
 			continue;
 		}
+		if (chN.isNull()) { // If one of the children evaluated to a null object
+			continue;
+		}		
 		if (dim == 2) {
 			CGAL_Nef_polyhedron2::Explorer explorer = chN.p2->explorer();
 			BOOST_FOREACH(const CGAL_Nef_polyhedron2::Explorer::Vertex &vh, 
@@ -151,14 +159,8 @@ CGAL_Nef_polyhedron CGALEvaluator::applyHull(const CgaladvNode &node)
 			}
 		}
 		else if (dim == 3) {
-			CGAL_Polyhedron P;
-			if (!chN.p3->is_simple()) {
-				PRINT("Hull() currently requires a valid 2-manifold. Please modify your design. See http://en.wikibooks.org/wiki/OpenSCAD_User_Manual/STL_Import_and_Export");
-			}
-			else {
-				chN.p3->convert_to_Polyhedron(P);
-				std::transform(P.vertices_begin(), P.vertices_end(), std::back_inserter(points3d), 
-											 boost::bind(static_cast<const CGAL_Polyhedron::Vertex::Point_3&(CGAL_Polyhedron::Vertex::*)() const>(&CGAL_Polyhedron::Vertex::point), _1));
+			for (CGAL_Nef_polyhedron3::Vertex_const_iterator i = chN.p3->vertices_begin(); i != chN.p3->vertices_end(); ++i) {
+				points3d.push_back(i->point());
 			}
 		}
 		chnode->progress_report();
@@ -171,14 +173,100 @@ CGAL_Nef_polyhedron CGALEvaluator::applyHull(const CgaladvNode &node)
 																										 CGAL_Nef_polyhedron2::INCLUDED));
 	}
 	else if (dim == 3) {
-		CGAL_Polyhedron P;
-		if (points3d.size()>3)
-			CGAL::convex_hull_3(points3d.begin(), points3d.end(), P);
-		N = CGAL_Nef_polyhedron(new CGAL_Nef_polyhedron3(P));
+		if (points3d.size() > 3) {
+
+    // Remove all duplicated points (speeds up the convex_hull computation significantly)
+			std::vector<CGAL_Polyhedron::Vertex::Point_3> unique_points;
+			Grid3d<int> grid(GRID_FINE);
+			
+			BOOST_FOREACH(CGAL_Polyhedron::Vertex::Point_3 const& p, points3d) {
+				double x = to_double(p.x()), y = to_double(p.y()), z = to_double(p.z());
+				int& v = grid.align(x,y,z);
+				if (v == 0) {
+					unique_points.push_back(CGAL_Polyhedron::Vertex::Point_3(x,y,z));
+					v = 1;
+				}
+			}
+
+			// Apply hull
+			if (points3d.size() >= 4) {
+				CGAL_Polyhedron P;
+				CGAL::convex_hull_3(unique_points.begin(), unique_points.end(), P);
+				N = CGAL_Nef_polyhedron(new CGAL_Nef_polyhedron3(P));
+			}
+		}
 	}
 	return N;
 }
 
+CGAL_Nef_polyhedron CGALEvaluator::applyResize(const CgaladvNode &node)
+{
+	// Based on resize() in Giles Bathgate's RapCAD (but not exactly)
+	CGAL_Nef_polyhedron N;
+	N = applyToChildren(node, CGE_UNION);
+
+	if ( N.isNull() || N.isEmpty() ) return N;
+
+	for (int i=0;i<3;i++) {
+		if (node.newsize[i]<0) {
+			PRINT("WARNING: Cannot resize to sizes less than 0.");
+			return N;
+		}
+	}
+
+	CGAL_Iso_cuboid_3 bb;
+
+	if ( N.dim == 2 ) {
+		CGAL_Iso_rectangle_2e bbox = bounding_box( *N.p2 );
+		CGAL_Point_2e min2(bbox.min()), max2(bbox.max());
+		CGAL_Point_3 min3(CGAL::to_double(min2.x()), CGAL::to_double(min2.y()), 0),
+			max3(CGAL::to_double(max2.x()), CGAL::to_double(max2.y()), 0);
+		bb = CGAL_Iso_cuboid_3( min3, max3 );
+	}
+	else {
+		bb = bounding_box( *N.p3 );
+	}
+
+	std::vector<NT3> scale, bbox_size;
+	for (int i=0;i<3;i++) scale.push_back( NT3(1) );
+	bbox_size.push_back( bb.xmax()-bb.xmin() );
+	bbox_size.push_back( bb.ymax()-bb.ymin() );
+	bbox_size.push_back( bb.zmax()-bb.zmin() );
+	int newsizemax_index = 0;
+	for (int i=0;i<N.dim;i++) {
+		if (node.newsize[i]) {
+			if (bbox_size[i]==NT3(0)) {
+				PRINT("WARNING: Resize in direction normal to flat object is not implemented");
+				return N;
+			}
+			else {
+				scale[i] = NT3(node.newsize[i]) / bbox_size[i];
+			}
+			if ( node.newsize[i] > node.newsize[newsizemax_index] )
+				newsizemax_index = i;
+		}
+	}
+
+	NT3 autoscale = NT3( 1 );
+	if ( node.newsize[ newsizemax_index ] != 0 )
+		autoscale = NT3( node.newsize[ newsizemax_index ] ) / bbox_size[ newsizemax_index ];
+	for (int i=0;i<N.dim;i++) {
+		if (node.autosize[i] && node.newsize[i]==0)
+			scale[i] = autoscale;
+	}
+
+	Eigen::Matrix4d t;
+	t << CGAL::to_double(scale[0]),           0,        0,        0,
+	     0,        CGAL::to_double(scale[1]),           0,        0,
+	     0,        0,        CGAL::to_double(scale[2]),           0,
+	     0,        0,        0,                                   1;
+
+	N.transform( Transform3d( t ) );
+	return N;
+}
+
+
+
 /*
 	Typical visitor behavior:
 	o In prefix: Check if we're cached -> prune
@@ -216,7 +304,7 @@ Response CGALEvaluator::visit(State &state, const CsgNode &node)
 	if (state.isPostfix()) {
 		CGAL_Nef_polyhedron N;
 		if (!isCached(node)) {
-			CGALEvaluator::CsgOp op;
+			CGALEvaluator::CsgOp op = CGE_UNION;
 			switch (node.type) {
 			case CSG_TYPE_UNION:
 				op = CGE_UNION;
@@ -253,57 +341,7 @@ Response CGALEvaluator::visit(State &state, const TransformNode &node)
 				PRINT("Warning: Transformation matrix contains Not-a-Number and/or Infinity - removing object.");
 				N.reset();
 			}
-
-			// Then apply transform
-			// If there is no geometry under the transform, N will be empty
-			// just silently ignore such nodes
-			if (!N.isNull()) {
-				if (N.dim == 2) {
-					// Unfortunately CGAL provides no transform method for CGAL_Nef_polyhedron2
-					// objects. So we convert in to our internal 2d data format, transform it,
-					// tesselate it and create a new CGAL_Nef_polyhedron2 from it.. What a hack!
-					
-					Eigen::Matrix2f testmat;
-					testmat << node.matrix(0,0), node.matrix(0,1), node.matrix(1,0), node.matrix(1,1);
-					if (testmat.determinant() == 0) {
-						PRINT("Warning: Scaling a 2D object with 0 - removing object");
-						N.reset();
-					}
-					else {
-						CGAL_Aff_transformation2 t(
-							node.matrix(0,0), node.matrix(0,1), node.matrix(0,3),
-							node.matrix(1,0), node.matrix(1,1), node.matrix(1,3), node.matrix(3,3));
-						
-						DxfData *dd = N.convertToDxfData();
-						for (size_t i=0; i < dd->points.size(); i++) {
-							CGAL_Kernel2::Point_2 p = CGAL_Kernel2::Point_2(dd->points[i][0], dd->points[i][1]);
-							p = t.transform(p);
-							dd->points[i][0] = to_double(p.x());
-							dd->points[i][1] = to_double(p.y());
-						}
-						
-						PolySet ps;
-						ps.is2d = true;
-						dxf_tesselate(&ps, *dd, 0, true, false, 0);
-						
-						N = evaluateCGALMesh(ps);
-						delete dd;
-					}
-				}
-				else if (N.dim == 3) {
-					if (node.matrix.matrix().determinant() == 0) {
-						PRINT("Warning: Scaling a 3D object with 0 - removing object");
-						N.reset();
-					}
-					else {
-						CGAL_Aff_transformation t(
-							node.matrix(0,0), node.matrix(0,1), node.matrix(0,2), node.matrix(0,3),
-							node.matrix(1,0), node.matrix(1,1), node.matrix(1,2), node.matrix(1,3),
-							node.matrix(2,0), node.matrix(2,1), node.matrix(2,2), node.matrix(2,3), node.matrix(3,3));
-						N.p3->transform(t);
-					}
-				}
-			}
+			N.transform( node.matrix );
 		}
 		else {
 			N = CGALCache::instance()->get(this->tree.getIdString(node));
@@ -358,6 +396,9 @@ Response CGALEvaluator::visit(State &state, const CgaladvNode &node)
 			case HULL:
 				N = applyHull(node);
 				break;
+			case RESIZE:
+				N = applyResize(node);
+				break;
 			}
 		}
 		else {
@@ -644,19 +685,35 @@ CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const PolySet &ps)
 	else // not (this->is2d)
 	{
 		CGAL_Nef_polyhedron3 *N = NULL;
+		bool plane_error = false;
 		CGAL::Failure_behaviour old_behaviour = CGAL::set_error_behaviour(CGAL::THROW_EXCEPTION);
 		try {
-			// FIXME: Are we leaking memory for the CGAL_Polyhedron object?
-			CGAL_Polyhedron *P = createPolyhedronFromPolySet(ps);
-			if (P) {
-				N = new CGAL_Nef_polyhedron3(*P);
-			}
+			CGAL_Polyhedron P;
+			bool err = createPolyhedronFromPolySet(ps,P);
+			if (!err) N = new CGAL_Nef_polyhedron3(P);
 		}
 		catch (const CGAL::Assertion_exception &e) {
-			PRINTB("CGAL error in CGAL_Nef_polyhedron3(): %s", e.what());
+			if (std::string(e.what()).find("Plane_constructor")!=std::string::npos) {
+				if (std::string(e.what()).find("has_on")!=std::string::npos) {
+					PRINT("PolySet has nonplanar faces. Attempting alternate construction");
+					plane_error=true;
+				}
+			} else {
+				PRINTB("CGAL error in CGAL_Nef_polyhedron3(): %s", e.what());
+			}
+		}
+		if (plane_error) try {
+			PolySet ps2;
+			CGAL_Polyhedron P;
+			tessellate_3d_faces( ps, ps2 );
+			bool err = createPolyhedronFromPolySet(ps2,P);
+			if (!err) N = new CGAL_Nef_polyhedron3(P);
+		}
+		catch (const CGAL::Assertion_exception &e) {
+			PRINTB("Alternate construction failed. CGAL error in CGAL_Nef_polyhedron3(): %s", e.what());
 		}
 		CGAL::set_error_behaviour(old_behaviour);
-		return CGAL_Nef_polyhedron(N);
+		if (N) return CGAL_Nef_polyhedron(N);
 	}
-	return CGAL_Nef_polyhedron();
+	return CGAL_Nef_polyhedron(ps.is2d?2:3);
 }
