@@ -4,12 +4,13 @@
 #include <QMainWindow>
 #include "ui_MainWindow.h"
 #include "openscad.h"
-#include "context.h"
+#include "modcontext.h"
 #include "module.h"
 #include "Tree.h"
 #include "memory.h"
 #include <vector>
 #include <QMutex>
+#include <QSet>
 
 class MainWindow : public QMainWindow, public Ui::MainWindow
 {
@@ -19,7 +20,6 @@ public:
 	static void requestOpenFile(const QString &filename);
 
 	QString fileName;
-	class Highlighter *highlighter;
 
 	class Preferences *prefs;
 
@@ -28,9 +28,10 @@ public:
 
 	QTimer *autoReloadTimer;
 	std::string autoReloadId;
+	QTimer *waitAfterReloadTimer;
 
-	Context root_ctx;
-	Module *root_module;      // Result of parsing
+	ModuleContext top_ctx;
+	FileModule *root_module;      // Result of parsing
 	ModuleInstantiation root_inst;    // Top level instance
 	AbstractNode *absolute_root_node; // Result of tree evaluation
 	AbstractNode *root_node;          // Root if the root modifier (!) is used
@@ -53,6 +54,7 @@ public:
 	std::vector<shared_ptr<CSGTerm> > background_terms;
 	CSGChain *background_chain;
 	QString last_compiled_doc;
+	static QString qexamplesdir;
 
 	static const int maxRecentFiles = 10;
 	QAction *actionRecentFile[maxRecentFiles];
@@ -74,12 +76,10 @@ private slots:
 private:
 	void openFile(const QString &filename);
 	void refreshDocument();
-	AbstractNode *find_root_tag(AbstractNode *n);
 	void updateTemporalVariables();
 	bool fileChangedOnDisk();
-	bool includesChanged();
-	bool compileTopLevelDocument(bool reload);
-	bool compile(bool reload, bool procevents);
+	void compileTopLevelDocument();
+	void compile(bool reload, bool forcedone = false);
 	void compileCSG(bool procevents);
 	bool maybeSave();
 	bool checkEditorModified();
@@ -87,10 +87,13 @@ private:
 	static void consoleOutput(const std::string &msg, void *userdata);
 	void loadViewSettings();
 	void loadDesignSettings();
+	void saveBackup();
+	void writeBackup(class QFile *file);
 
   class QMessageBox *openglbox;
 
 private slots:
+	void actionUpdateCheck();
 	void actionNew();
 	void actionOpen();
 	void actionOpenRecent();
@@ -101,6 +104,11 @@ private slots:
 	void actionSave();
 	void actionSaveAs();
 	void actionReload();
+	void actionShowLibraryFolder();
+
+	void instantiateRoot();
+	void compileDone(bool didchange);
+	void compileEnded();
 
 private slots:
 	void pasteViewportTranslation();
@@ -109,11 +117,28 @@ private slots:
 	void preferences();
 
 private slots:
-	void actionCompile();
+	void actionRenderCSG();
+	void selectFindType(int);
+	void find();
+	void findAndReplace();
+	void findNext();
+	void findPrev();
+	void useSelectionForFind();
+	void replace();
+	void replaceAll();
+protected:
+	bool findOperation(QTextDocument::FindFlags options = 0);
+	virtual bool eventFilter(QObject* obj, QEvent *event);
+
+private slots:
+	void csgRender();
+	void csgReloadRender();
 #ifdef ENABLE_CGAL
 	void actionRenderCGAL();
 	void actionRenderCGALDone(class CGAL_Nef_polyhedron *);
+	void cgalRender();
 #endif
+	void actionCheckValidity();
 	void actionDisplayAST();
 	void actionDisplayCSGTree();
 	void actionDisplayCSGProducts();
@@ -126,11 +151,14 @@ private slots:
 	void actionFlushCaches();
 
 public:
+	static QSet<MainWindow*> *windows;
+	static void setExamplesDir(const QString &dir) { MainWindow::qexamplesdir = dir; }
 	void viewModeActionsUncheck();
 	void setCurrentOutput();
 	void clearCurrentOutput();
 
 public slots:
+	void actionReloadRenderCSG();
 #ifdef ENABLE_OPENCSG
 	void viewModeOpenCSG();
 #endif
@@ -153,6 +181,7 @@ public slots:
 	void viewCenter();
 	void viewPerspective();
 	void viewOrthogonal();
+        void viewResetView();
 	void hideConsole();
 	void animateUpdateDocChanged();
 	void animateUpdate();
@@ -163,16 +192,23 @@ public slots:
 	void helpManual();
 	void helpLibrary();
 	void quit();
-	void actionReloadCompile();
 	void checkAutoReload();
+	void waitAfterReload();
 	void autoReloadSet(bool);
 
 private:
 	static void report_func(const class AbstractNode*, void *vp, int mark);
+	
+	char const * afterCompileSlot;
+	bool procevents;
+	class QTemporaryFile *tempFile;
 
 	class ProgressWidget *progresswidget;
 	class CGALWorker *cgalworker;
 	QMutex consolemutex;
+signals:
+	void highlightError(int);
+	void unhighlightLastError();
 };
 
 class GuiLocker
@@ -189,7 +225,7 @@ public:
 	static void unlock() { gui_locked--; }
 
 private:
-	static unsigned int gui_locked;
+ 	static unsigned int gui_locked;
 };
 
 #endif
