@@ -1,11 +1,16 @@
 #include <string>
 #include <boost/unordered_map.hpp>
 
+/*!
+	Caches FileModules based on their filenames
+*/
 class ModuleCache
 {
 public:
 	static ModuleCache *instance() { if (!inst) inst = new ModuleCache; return inst; }
-	class Module *evaluate(const std::string &filename);
+	bool evaluate(const std::string &filename, class FileModule *&module);
+	class FileModule *lookup(const std::string &filename);
+	bool isCached(const std::string &filename);
 	size_t size() { return this->entries.size(); }
 	void clear();
 
@@ -16,7 +21,7 @@ private:
 	static ModuleCache *inst;
 
 	struct cache_entry {
-		class Module *module;
+		class FileModule *module;
 		std::string cache_id;
 	};
 	boost::unordered_map<std::string, cache_entry> entries;
