@@ -13,111 +13,13 @@
 #include "dxfdata.h"
 #include "dxftess.h"
 #include "module.h"
+#include "calc.h"
 
 #include "svg.h"
 #include "printutils.h"
-#include "openscad.h" // get_fragments_from_r()
 #include <boost/foreach.hpp>
 #include <vector>
 
-/*
-
-ZRemover
-
-This class converts one or more already 'flat' Nef3 polyhedra into a Nef2
-polyhedron by stripping off the 'z' coordinates from the vertices. The
-resulting Nef2 poly is accumulated in the 'output_nefpoly2d' member variable.
-
-The 'z' coordinates will either be all 0s, for an xy-plane intersected Nef3,
-or, they will be a mixture of -eps and +eps, for a thin-box intersected Nef3.
-
-Notes on CGAL's Nef Polyhedron2:
-
-1. The 'mark' on a 2d Nef face is important when doing unions/intersections.
- If the 'mark' of a face is wrong the resulting nef2 poly will be unexpected.
-2. The 'mark' can be dependent on the points fed to the Nef2 constructor.
- This is why we iterate through the 3d faces using the halfedge cycle
- source()->target() instead of the ordinary source()->source(). The
- the latter can generate sequences of points that will fail the
- the CGAL::is_simple_2() test, resulting in improperly marked nef2 polys.
-3. 3d facets have 'two sides'. we throw out the 'down' side to prevent dups.
-
-The class uses the 'visitor' pattern from the CGAL manual. See also
-http://www.cgal.org/Manual/latest/doc_html/cgal_manual/Nef_3/Chapter_main.html
-http://www.cgal.org/Manual/latest/doc_html/cgal_manual/Nef_3_ref/Class_Nef_polyhedron3.html
-OGL_helper.h
-*/
-
-class ZRemover {
-public:
-	logstream log;
-	CGAL_Nef_polyhedron2::Boundary boundary;
-	shared_ptr<CGAL_Nef_polyhedron2> tmpnef2d;
-	shared_ptr<CGAL_Nef_polyhedron2> output_nefpoly2d;
-	CGAL::Direction_3<CGAL_Kernel3> up;
-	ZRemover()
-	{
-		output_nefpoly2d.reset( new CGAL_Nef_polyhedron2() );
-		boundary = CGAL_Nef_polyhedron2::INCLUDED;
-		up = CGAL::Direction_3<CGAL_Kernel3>(0,0,1);
-		log = logstream(5);
-	}
-	void visit( CGAL_Nef_polyhedron3::Vertex_const_handle ) {}
-	void visit( CGAL_Nef_polyhedron3::Halfedge_const_handle ) {}
-	void visit( CGAL_Nef_polyhedron3::SHalfedge_const_handle ) {}
-	void visit( CGAL_Nef_polyhedron3::SHalfloop_const_handle ) {}
-	void visit( CGAL_Nef_polyhedron3::SFace_const_handle ) {}
-	void visit( CGAL_Nef_polyhedron3::Halffacet_const_handle hfacet ) {
-		log << " <!-- Halffacet visit. Mark: " << hfacet->mark() << " -->\n";
-		if ( hfacet->plane().orthogonal_direction() != this->up ) {
-			log << "  <!-- down-facing half-facet. skipping -->\n";
-			log << " <!-- Halffacet visit end-->\n";
-			return;
-		}
-
-		// possible optimization - throw out facets that are 'side facets' between
-		// the top & bottom of the big thin box. (i.e. mixture of z=-eps and z=eps)
-
-		CGAL_Nef_polyhedron3::Halffacet_cycle_const_iterator fci;
-		int contour_counter = 0;
-		CGAL_forall_facet_cycles_of( fci, hfacet ) {
-			if ( fci.is_shalfedge() ) {
-				CGAL_Nef_polyhedron3::SHalfedge_around_facet_const_circulator c1(fci), cend(c1);
-				std::vector<CGAL_Nef_polyhedron2::Explorer::Point> contour;
-				CGAL_For_all( c1, cend ) {
-					CGAL_Nef_polyhedron3::Point_3 point3d = c1->source()->target()->point();
-					CGAL_Nef_polyhedron2::Explorer::Point point2d( point3d.x(), point3d.y() );
-					contour.push_back( point2d );
-				}
-
-				if (contour.size()==0) continue;
-
-				log << " <!-- is_simple_2:" << CGAL::is_simple_2( contour.begin(), contour.end() ) << " --> \n";
-
-				tmpnef2d.reset( new CGAL_Nef_polyhedron2( contour.begin(), contour.end(), boundary ) );
-
-				if ( contour_counter == 0 ) {
-					log << " <!-- contour is a body. make union(). " << contour.size() << " points. -->\n" ;
-					*(output_nefpoly2d) += *(tmpnef2d);
-				} else {
-					log << " <!-- contour is a hole. make intersection(). " << contour.size() << " points. -->\n";
-					*(output_nefpoly2d) *= *(tmpnef2d);
-				}
-
-				log << "\n<!-- ======== output tmp nef: ==== -->\n"
-					<< OpenSCAD::dump_svg( *tmpnef2d ) << "\n"
-					<< "\n<!-- ======== output accumulator: ==== -->\n"
-					<< OpenSCAD::dump_svg( *output_nefpoly2d ) << "\n";
-
-				contour_counter++;
-			} else {
-				log << " <!-- trivial facet cycle skipped -->\n";
-			}
-		} // next facet cycle (i.e. next contour)
-		log << " <!-- Halffacet visit end -->\n";
-	} // visit()
-};
-
 PolySetCGALEvaluator::PolySetCGALEvaluator(CGALEvaluator &cgalevaluator)
 	: PolySetEvaluator(cgalevaluator.getTree()), cgalevaluator(cgalevaluator)
 {
@@ -186,24 +88,23 @@ PolySet *PolySetCGALEvaluator::evaluatePolySet(const ProjectionNode &node)
 			return NULL;
 		}
 
-		// remove z coordinates to make CGAL_Nef_polyhedron2
 		log << OpenSCAD::svg_header( 480, 100000 ) << "\n";
 		try {
-			ZRemover zremover;
-			CGAL_Nef_polyhedron3::Volume_const_iterator i;
-			CGAL_Nef_polyhedron3::Shell_entry_const_iterator j;
-			CGAL_Nef_polyhedron3::SFace_const_handle sface_handle;
-			for ( i = sum.p3->volumes_begin(); i != sum.p3->volumes_end(); ++i ) {
-				log << "<!-- volume. mark: " << i->mark() << " -->\n";
-				for ( j = i->shells_begin(); j != i->shells_end(); ++j ) {
-					log << "<!-- shell. mark: " << i->mark() << " -->\n";
-					sface_handle = CGAL_Nef_polyhedron3::SFace_const_handle( j );
-					sum.p3->visit_shell_objects( sface_handle , zremover );
-					log << "<!-- shell. end. -->\n";
-				}
-				log << "<!-- volume end. -->\n";
-			}
-			nef_poly.p2 = zremover.output_nefpoly2d;
+      ZRemover zremover;
+      CGAL_Nef_polyhedron3::Volume_const_iterator i;
+      CGAL_Nef_polyhedron3::Shell_entry_const_iterator j;
+      CGAL_Nef_polyhedron3::SFace_const_handle sface_handle;
+      for ( i = sum.p3->volumes_begin(); i != sum.p3->volumes_end(); ++i ) {
+        log << "<!-- volume. mark: " << i->mark() << " -->\n";
+        for ( j = i->shells_begin(); j != i->shells_end(); ++j ) {
+          log << "<!-- shell. mark: " << i->mark() << " -->\n";
+          sface_handle = CGAL_Nef_polyhedron3::SFace_const_handle( j );
+          sum.p3->visit_shell_objects( sface_handle , zremover );
+          log << "<!-- shell. end. -->\n";
+        }
+        log << "<!-- volume end. -->\n";
+      }
+      nef_poly.p2 = zremover.output_nefpoly2d;
 		}	catch (const CGAL::Failure_exception &e) {
 			PRINTB("CGAL error in projection node while flattening: %s", e.what());
 		}
@@ -301,27 +202,38 @@ PolySet *PolySetCGALEvaluator::evaluatePolySet(const ProjectionNode &node)
 	return ps;
 }
 
-static void add_slice(PolySet *ps, const DxfData &dxf, DxfData::Path &path, double rot1, double rot2, double h1, double h2)
+static void add_slice(PolySet *ps, const DxfData &dxf, DxfData::Path &path, 
+											double rot1, double rot2, 
+											double h1, double h2, 
+											double scale1_x, double scale1_y,
+											double scale2_x, double scale2_y)
 {
+	// FIXME: If scale2 == 0 we need to handle tessellation separately
 	bool splitfirst = sin(rot2 - rot1) >= 0.0;
-	for (size_t j = 1; j < path.indices.size(); j++)
-	{
+	for (size_t j = 1; j < path.indices.size(); j++) {
 		int k = j - 1;
 
-		double jx1 = dxf.points[path.indices[j]][0] *  cos(rot1*M_PI/180) + dxf.points[path.indices[j]][1] * sin(rot1*M_PI/180);
-		double jy1 = dxf.points[path.indices[j]][0] * -sin(rot1*M_PI/180) + dxf.points[path.indices[j]][1] * cos(rot1*M_PI/180);
+		double jx1 = scale1_x * (dxf.points[path.indices[j]][0] *  cos(rot1*M_PI/180) + 
+													 dxf.points[path.indices[j]][1] * sin(rot1*M_PI/180));
+		double jy1 = scale1_y * (dxf.points[path.indices[j]][0] * -sin(rot1*M_PI/180) + 
+													 dxf.points[path.indices[j]][1] * cos(rot1*M_PI/180));
 
-		double jx2 = dxf.points[path.indices[j]][0] *  cos(rot2*M_PI/180) + dxf.points[path.indices[j]][1] * sin(rot2*M_PI/180);
-		double jy2 = dxf.points[path.indices[j]][0] * -sin(rot2*M_PI/180) + dxf.points[path.indices[j]][1] * cos(rot2*M_PI/180);
+		double jx2 = scale2_x * (dxf.points[path.indices[j]][0] *  cos(rot2*M_PI/180) + 
+													 dxf.points[path.indices[j]][1] * sin(rot2*M_PI/180));
+		double jy2 = scale2_y * (dxf.points[path.indices[j]][0] * -sin(rot2*M_PI/180) + 
+													 dxf.points[path.indices[j]][1] * cos(rot2*M_PI/180));
 
-		double kx1 = dxf.points[path.indices[k]][0] *  cos(rot1*M_PI/180) + dxf.points[path.indices[k]][1] * sin(rot1*M_PI/180);
-		double ky1 = dxf.points[path.indices[k]][0] * -sin(rot1*M_PI/180) + dxf.points[path.indices[k]][1] * cos(rot1*M_PI/180);
+		double kx1 = scale1_x * (dxf.points[path.indices[k]][0] *  cos(rot1*M_PI/180) + 
+													 dxf.points[path.indices[k]][1] * sin(rot1*M_PI/180));
+		double ky1 = scale1_y * (dxf.points[path.indices[k]][0] * -sin(rot1*M_PI/180) + 
+													 dxf.points[path.indices[k]][1] * cos(rot1*M_PI/180));
 
-		double kx2 = dxf.points[path.indices[k]][0] *  cos(rot2*M_PI/180) + dxf.points[path.indices[k]][1] * sin(rot2*M_PI/180);
-		double ky2 = dxf.points[path.indices[k]][0] * -sin(rot2*M_PI/180) + dxf.points[path.indices[k]][1] * cos(rot2*M_PI/180);
+		double kx2 = scale2_x * (dxf.points[path.indices[k]][0] *  cos(rot2*M_PI/180) + 
+													 dxf.points[path.indices[k]][1] * sin(rot2*M_PI/180));
+		double ky2 = scale2_y * (dxf.points[path.indices[k]][0] * -sin(rot2*M_PI/180) + 
+													 dxf.points[path.indices[k]][1] * cos(rot2*M_PI/180));
 
-		if (splitfirst)
-		{
+		if (splitfirst) {
 			ps->append_poly();
 			if (path.is_inner) {
 				ps->append_vertex(kx1, ky1, h1);
@@ -333,19 +245,20 @@ static void add_slice(PolySet *ps, const DxfData &dxf, DxfData::Path &path, doub
 				ps->insert_vertex(jx2, jy2, h2);
 			}
 
-			ps->append_poly();
-			if (path.is_inner) {
-				ps->append_vertex(kx2, ky2, h2);
-				ps->append_vertex(kx1, ky1, h1);
-				ps->append_vertex(jx2, jy2, h2);
-			} else {
-				ps->insert_vertex(kx2, ky2, h2);
-				ps->insert_vertex(kx1, ky1, h1);
-				ps->insert_vertex(jx2, jy2, h2);
+			if (scale2_x > 0 || scale2_y > 0) {
+				ps->append_poly();
+				if (path.is_inner) {
+					ps->append_vertex(kx2, ky2, h2);
+					ps->append_vertex(kx1, ky1, h1);
+					ps->append_vertex(jx2, jy2, h2);
+				} else {
+					ps->insert_vertex(kx2, ky2, h2);
+					ps->insert_vertex(kx1, ky1, h1);
+					ps->insert_vertex(jx2, jy2, h2);
+				}
 			}
 		}
-		else
-		{
+		else {
 			ps->append_poly();
 			if (path.is_inner) {
 				ps->append_vertex(kx1, ky1, h1);
@@ -357,15 +270,17 @@ static void add_slice(PolySet *ps, const DxfData &dxf, DxfData::Path &path, doub
 				ps->insert_vertex(kx2, ky2, h2);
 			}
 
-			ps->append_poly();
-			if (path.is_inner) {
-				ps->append_vertex(jx2, jy2, h2);
-				ps->append_vertex(kx2, ky2, h2);
-				ps->append_vertex(jx1, jy1, h1);
-			} else {
-				ps->insert_vertex(jx2, jy2, h2);
-				ps->insert_vertex(kx2, ky2, h2);
-				ps->insert_vertex(jx1, jy1, h1);
+			if (scale2_x > 0 || scale2_y > 0) {
+				ps->append_poly();
+				if (path.is_inner) {
+					ps->append_vertex(jx2, jy2, h2);
+					ps->append_vertex(kx2, ky2, h2);
+					ps->append_vertex(jx1, jy1, h1);
+				} else {
+					ps->insert_vertex(jx2, jy2, h2);
+					ps->insert_vertex(kx2, ky2, h2);
+					ps->insert_vertex(jx1, jy1, h1);
+				}
 			}
 		}
 	}
@@ -397,7 +312,7 @@ PolySet *PolySetCGALEvaluator::evaluatePolySet(const LinearExtrudeNode &node)
 		if (sum.isNull()) return NULL;
 		dxf = sum.convertToDxfData();;
 	} else {
-		dxf = new DxfData(node.fn, node.fs, node.fa, node.filename, node.layername, node.origin_x, node.origin_y, node.scale);
+		dxf = new DxfData(node.fn, node.fs, node.fa, node.filename, node.layername, node.origin_x, node.origin_y, node.scale_x);
 	}
 
 	PolySet *ps = extrudeDxfData(node, *dxf);
@@ -409,6 +324,7 @@ PolySet *PolySetCGALEvaluator::extrudeDxfData(const LinearExtrudeNode &node, Dxf
 {
 	PolySet *ps = new PolySet();
 	ps->convexity = node.convexity;
+	if (node.height <= 0) return ps;
 
 	double h1, h2;
 
@@ -421,50 +337,49 @@ PolySet *PolySetCGALEvaluator::extrudeDxfData(const LinearExtrudeNode &node, Dxf
 	}
 
 	bool first_open_path = true;
-	for (size_t i = 0; i < dxf.paths.size(); i++)
-	{
-		if (dxf.paths[i].is_closed)
-			continue;
+	for (size_t i = 0; i < dxf.paths.size(); i++) {
+		if (dxf.paths[i].is_closed) continue;
 		if (first_open_path) {
 			PRINTB("WARNING: Open paths in dxf_linear_extrude(file = \"%s\", layer = \"%s\"):",
 					node.filename % node.layername);
 			first_open_path = false;
 		}
 		PRINTB("   %9.5f %10.5f ... %10.5f %10.5f",
-					 (dxf.points[dxf.paths[i].indices.front()][0] / node.scale + node.origin_x) %
-					 (dxf.points[dxf.paths[i].indices.front()][1] / node.scale + node.origin_y) %
-					 (dxf.points[dxf.paths[i].indices.back()][0] / node.scale + node.origin_x) %
-					 (dxf.points[dxf.paths[i].indices.back()][1] / node.scale + node.origin_y));
+					 (dxf.points[dxf.paths[i].indices.front()][0] / node.scale_x + node.origin_x) %
+					 (dxf.points[dxf.paths[i].indices.front()][1] / node.scale_y + node.origin_y) %
+					 (dxf.points[dxf.paths[i].indices.back()][0] / node.scale_x + node.origin_x) %
+					 (dxf.points[dxf.paths[i].indices.back()][1] / node.scale_y + node.origin_y));
 	}
 
 
-	if (node.has_twist)
-	{
-		dxf_tesselate(ps, dxf, 0, false, true, h1);
-		dxf_tesselate(ps, dxf, node.twist, true, true, h2);
-		for (int j = 0; j < node.slices; j++)
-		{
+	if (node.has_twist) {
+		dxf_tesselate(ps, dxf, 0, Vector2d(1,1), false, true, h1); // bottom
+		if (node.scale_x > 0 || node.scale_y > 0) {
+			dxf_tesselate(ps, dxf, node.twist, Vector2d(node.scale_x, node.scale_y), true, true, h2); // top
+		}
+		for (int j = 0; j < node.slices; j++) {
 			double t1 = node.twist*j / node.slices;
 			double t2 = node.twist*(j+1) / node.slices;
 			double g1 = h1 + (h2-h1)*j / node.slices;
 			double g2 = h1 + (h2-h1)*(j+1) / node.slices;
-			for (size_t i = 0; i < dxf.paths.size(); i++)
-			{
-				if (!dxf.paths[i].is_closed)
-					continue;
-				add_slice(ps, dxf, dxf.paths[i], t1, t2, g1, g2);
+			double s1x = 1 - (1-node.scale_x)*j / node.slices;
+			double s1y = 1 - (1-node.scale_y)*j / node.slices;
+			double s2x = 1 - (1-node.scale_x)*(j+1) / node.slices;
+			double s2y = 1 - (1-node.scale_y)*(j+1) / node.slices;
+			for (size_t i = 0; i < dxf.paths.size(); i++) {
+				if (!dxf.paths[i].is_closed) continue;
+				add_slice(ps, dxf, dxf.paths[i], t1, t2, g1, g2, s1x, s1y, s2x, s2y);
 			}
 		}
 	}
-	else
-	{
-		dxf_tesselate(ps, dxf, 0, false, true, h1);
-		dxf_tesselate(ps, dxf, 0, true, true, h2);
-		for (size_t i = 0; i < dxf.paths.size(); i++)
-		{
-			if (!dxf.paths[i].is_closed)
-				continue;
-			add_slice(ps, dxf, dxf.paths[i], 0, 0, h1, h2);
+	else {
+		dxf_tesselate(ps, dxf, 0, Vector2d(1,1), false, true, h1); //bottom
+		if (node.scale_x > 0 || node.scale_y > 0) {
+			dxf_tesselate(ps, dxf, 0, Vector2d(node.scale_x, node.scale_y), true, true, h2); // top
+		}
+		for (size_t i = 0; i < dxf.paths.size(); i++) {
+			if (!dxf.paths[i].is_closed) continue;
+			add_slice(ps, dxf, dxf.paths[i], 0, 0, h1, h2, 1, 1, node.scale_x, node.scale_y);
 		}
 	}
 
@@ -540,12 +455,21 @@ PolySet *PolySetCGALEvaluator::rotateDxfData(const RotateExtrudeNode &node, DxfD
 
 	for (size_t i = 0; i < dxf.paths.size(); i++)
 	{
+		double min_x = 0;
 		double max_x = 0;
 		for (size_t j = 0; j < dxf.paths[i].indices.size(); j++) {
-			max_x = fmax(max_x, dxf.points[dxf.paths[i].indices[j]][0]);
+			double point_x = dxf.points[dxf.paths[i].indices[j]][0];
+			min_x = fmin(min_x, point_x);
+			max_x = fmax(max_x, point_x);
+
+			if ((max_x - min_x) > max_x && (max_x - min_x) > fabs(min_x)) {
+				PRINTB("ERROR: all points for rotate_extrude() must have the same X coordinate sign (range is %.2f -> %.2f)", min_x % max_x);
+				delete ps;
+				return NULL;
+			}
 		}
 
-		int fragments = get_fragments_from_r(max_x, node.fn, node.fs, node.fa);
+		int fragments = Calc::get_fragments_from_r(max_x-min_x, node.fn, node.fs, node.fa);
 
 		double ***points;
 		points = new double**[fragments];
