@@ -3,6 +3,8 @@
 
 #include <string>
 #include <boost/unordered_map.hpp>
+#include "module.h"
+#include "localscope.h"
 
 class Builtins
 {
@@ -16,19 +18,15 @@ public:
 	void initialize();
 	std::string isDeprecated(const std::string &name);
 
-	const FunctionContainer &functions() { return this->builtinfunctions; }
-	const ModuleContainer &modules() { return this->builtinmodules; }
+	const LocalScope &getGlobalScope() { return this->globalscope; }
 
 private:
-	Builtins() { }
+	Builtins();
 	~Builtins();
 
-	FunctionContainer builtinfunctions;
-	ModuleContainer builtinmodules;
+	LocalScope globalscope;
 
 	boost::unordered_map<std::string, std::string> deprecations;
 };
 
-extern void register_builtin(class Context &ctx);
-
 #endif
