@@ -8,10 +8,9 @@
 
 #include <map>
 
-PolySet *createPolySetFromPolyhedron(const CGAL_Polyhedron &p)
+bool createPolySetFromPolyhedron(const CGAL_Polyhedron &p, PolySet &ps)
 {
-	PolySet *ps = new PolySet();
-		
+	bool err = false;
 	typedef CGAL_Polyhedron::Vertex                                 Vertex;
 	typedef CGAL_Polyhedron::Vertex_const_iterator                  VCI;
 	typedef CGAL_Polyhedron::Facet_const_iterator                   FCI;
@@ -35,13 +34,13 @@ PolySet *createPolySetFromPolyhedron(const CGAL_Polyhedron &p)
 			double x3 = CGAL::to_double(v3.point().x());
 			double y3 = CGAL::to_double(v3.point().y());
 			double z3 = CGAL::to_double(v3.point().z());
-			ps->append_poly();
-			ps->append_vertex(x1, y1, z1);
-			ps->append_vertex(x2, y2, z2);
-			ps->append_vertex(x3, y3, z3);
+			ps.append_poly();
+			ps.append_vertex(x1, y1, z1);
+			ps.append_vertex(x2, y2, z2);
+			ps.append_vertex(x3, y3, z3);
 		} while (hc != hc_end);
 	}
-	return ps;
+	return err;
 }
 
 #undef GEN_SURFACE_DEBUG
@@ -127,27 +126,25 @@ public:
 	}
 };
 
-CGAL_Polyhedron *createPolyhedronFromPolySet(const PolySet &ps)
+bool createPolyhedronFromPolySet(const PolySet &ps, CGAL_Polyhedron &p)
 {
-	CGAL_Polyhedron *P = NULL;
+	bool err = false;
 	CGAL::Failure_behaviour old_behaviour = CGAL::set_error_behaviour(CGAL::THROW_EXCEPTION);
 	try {
-		P = new CGAL_Polyhedron;
 		CGAL_Build_PolySet builder(ps);
-		P->delegate(builder);
+		p.delegate(builder);
 	}
 	catch (const CGAL::Assertion_exception &e) {
 		PRINTB("CGAL error in CGAL_Build_PolySet: %s", e.what());
-		delete P;
-		P = NULL;
+		err = true;
 	}
 	CGAL::set_error_behaviour(old_behaviour);
-	return P;
+	return err;
 }
 
 CGAL_Iso_cuboid_3 bounding_box( const CGAL_Nef_polyhedron3 &N )
 {
-	CGAL_Iso_cuboid_3 result(-1,-1,-1,1,1,1);
+	CGAL_Iso_cuboid_3 result(0,0,0,0,0,0);
 	CGAL_Nef_polyhedron3::Vertex_const_iterator vi;
 	std::vector<CGAL_Nef_polyhedron3::Point_3> points;
 	// can be optimized by rewriting bounding_box to accept vertices
@@ -160,7 +157,7 @@ CGAL_Iso_cuboid_3 bounding_box( const CGAL_Nef_polyhedron3 &N )
 
 CGAL_Iso_rectangle_2e bounding_box( const CGAL_Nef_polyhedron2 &N )
 {
-	CGAL_Iso_rectangle_2e result(-1,-1,1,1);
+	CGAL_Iso_rectangle_2e result(0,0,0,0);
 	CGAL_Nef_polyhedron2::Explorer explorer = N.explorer();
 	CGAL_Nef_polyhedron2::Explorer::Vertex_const_iterator vi;
 	std::vector<CGAL_Point_2e> points;
@@ -173,5 +170,57 @@ CGAL_Iso_rectangle_2e bounding_box( const CGAL_Nef_polyhedron2 &N )
 	return result;
 }
 
+void ZRemover::visit( CGAL_Nef_polyhedron3::Halffacet_const_handle hfacet )
+{
+	log << " <!-- ZRemover Halffacet visit. Mark: " << hfacet->mark() << " -->\n";
+	if ( hfacet->plane().orthogonal_direction() != this->up ) {
+		log << "  <!-- ZRemover down-facing half-facet. skipping -->\n";
+		log << " <!-- ZRemover Halffacet visit end-->\n";
+		return;
+	}
+
+	// possible optimization - throw out facets that are vertically oriented
+
+	CGAL_Nef_polyhedron3::Halffacet_cycle_const_iterator fci;
+	int contour_counter = 0;
+	CGAL_forall_facet_cycles_of( fci, hfacet ) {
+		if ( fci.is_shalfedge() ) {
+			log << " <!-- ZRemover Halffacet cycle begin -->\n";
+			CGAL_Nef_polyhedron3::SHalfedge_around_facet_const_circulator c1(fci), cend(c1);
+			std::vector<CGAL_Nef_polyhedron2::Explorer::Point> contour;
+			CGAL_For_all( c1, cend ) {
+				CGAL_Nef_polyhedron3::Point_3 point3d = c1->source()->target()->point();
+				CGAL_Nef_polyhedron2::Explorer::Point point2d(CGAL::to_double(point3d.x()),
+																											CGAL::to_double(point3d.y()));
+				contour.push_back( point2d );
+			}
+			if (contour.size()==0) continue;
+
+			log << " <!-- is_simple_2:" << CGAL::is_simple_2( contour.begin(), contour.end() ) << " --> \n";
+
+			tmpnef2d.reset( new CGAL_Nef_polyhedron2( contour.begin(), contour.end(), boundary ) );
+
+			if ( contour_counter == 0 ) {
+				log << " <!-- contour is a body. make union(). " << contour.size() << " points. -->\n" ;
+				*(output_nefpoly2d) += *(tmpnef2d);
+			} else {
+				log << " <!-- contour is a hole. make intersection(). " << contour.size() << " points. -->\n";
+				*(output_nefpoly2d) *= *(tmpnef2d);
+			}
+
+			/*log << "\n<!-- ======== output tmp nef: ==== -->\n"
+				<< OpenSCAD::dump_svg( *tmpnef2d ) << "\n"
+				<< "\n<!-- ======== output accumulator: ==== -->\n"
+				<< OpenSCAD::dump_svg( *output_nefpoly2d ) << "\n";*/
+
+			contour_counter++;
+		} else {
+			log << " <!-- ZRemover trivial facet cycle skipped -->\n";
+		}
+		log << " <!-- ZRemover Halffacet cycle end -->\n";
+	}
+	log << " <!-- ZRemover Halffacet visit end -->\n";
+}
+
 #endif /* ENABLE_CGAL */
 
