@@ -2,28 +2,16 @@
 #include <QString>
 #include <QWidget>
 #include <QWheelEvent>
-
-#ifdef _QCODE_EDIT_
-#include <qeditor.h>
-class Editor : public QEditor
-#else
+#include <QScrollBar>
 #include <QTextEdit>
+#include "highlighter.h"
+
 class Editor : public QTextEdit
-#endif
 {
 	Q_OBJECT
 public:
-#ifdef _QCODE_EDIT_
-	Editor(QWidget *parent) : QEditor(parent) {}
-	QString toPlainText() const { return text(); }
-	void setPlainText(const QString& text) { setText(text); }
-public slots:
-	//void zoomIn() { zoom(1); }
-	void zoomIn(int n = 1) { zoom(n); }
-	//void zoomOut() { zoom(-1); } 
-	void zoomOut(int n = 1) { zoom(-n); } 
-#else
-	Editor(QWidget *parent) : QTextEdit(parent) { setAcceptRichText(false); }
+	Editor(QWidget *parent);
+	~Editor();
 public slots:
 	void zoomIn();
 	void zoomOut();
@@ -34,7 +22,11 @@ public slots:
 	void unindentSelection();
 	void commentSelection();
 	void uncommentSelection();
+	void setPlainText(const QString &text);
+	void highlightError(int error_pos);
+	void unhighlightLastError();
+	void setHighlightScheme(const QString &name);
 private:
 	void wheelEvent ( QWheelEvent * event );
-#endif
+	Highlighter *highlighter;
 };
