@@ -26,19 +26,34 @@
 
 #include "expression.h"
 #include "value.h"
-#include "context.h"
+#include "evalcontext.h"
 #include <assert.h>
 #include <sstream>
 #include <algorithm>
 #include "stl-utils.h"
+#include "printutils.h"
 #include <boost/bind.hpp>
 #include <boost/foreach.hpp>
 
-Expression::Expression()
+Expression::Expression() : recursioncount(0)
 {
 }
 
-Expression::Expression(const Value &val) : const_value(val), type("C")
+Expression::Expression(const std::string &type, 
+											 Expression *left, Expression *right)
+	: type(type), recursioncount(0)
+{
+	this->children.push_back(left);
+	this->children.push_back(right);
+}
+
+Expression::Expression(const std::string &type, Expression *expr)
+	: type(type), recursioncount(0)
+{
+	this->children.push_back(expr);
+}
+
+Expression::Expression(const Value &val) : const_value(val), type("C"), recursioncount(0)
 {
 }
 
@@ -47,6 +62,18 @@ Expression::~Expression()
 	std::for_each(this->children.begin(), this->children.end(), del_fun<Expression>());
 }
 
+class FuncRecursionGuard
+{
+public:
+	FuncRecursionGuard(const Expression &e) : expr(e) { 
+		expr.recursioncount++; 
+	}
+	~FuncRecursionGuard() { expr.recursioncount--; }
+	bool recursion_detected() const { return (expr.recursioncount > 1000); }
+private:
+	const Expression &expr;
+};
+
 Value Expression::evaluate(const Context *context) const
 {
 	if (this->type == "!")
@@ -78,8 +105,7 @@ Value Expression::evaluate(const Context *context) const
 	if (this->type == ">")
 		return this->children[0]->evaluate(context) > this->children[1]->evaluate(context);
 	if (this->type == "?:") {
-		Value v = this->children[0]->evaluate(context);
-		return this->children[v.toBool() ? 1 : 2]->evaluate(context);
+		return this->children[this->children[0]->evaluate(context) ? 1 : 2]->evaluate(context);
 	}
 	if (this->type == "[]") {
 		return this->children[0]->evaluate(context)[this->children[1]->evaluate(context)];
@@ -91,10 +117,18 @@ Value Expression::evaluate(const Context *context) const
 	if (this->type == "R") {
 		Value v1 = this->children[0]->evaluate(context);
 		Value v2 = this->children[1]->evaluate(context);
-		Value v3 = this->children[2]->evaluate(context);
-		if (v1.type() == Value::NUMBER && v2.type() == Value::NUMBER && v3.type() == Value::NUMBER) {
-			return Value(v1.toDouble(), v2.toDouble(), v3.toDouble());
-		}
+                if (this->children.size() == 2) {
+                        if (v1.type() == Value::NUMBER && v2.type() == Value::NUMBER) {
+                                Value::RangeType range(v1.toDouble(), v2.toDouble());
+                                return Value(range);
+                        }
+                } else {
+        		Value v3 = this->children[2]->evaluate(context);
+                        if (v1.type() == Value::NUMBER && v2.type() == Value::NUMBER && v3.type() == Value::NUMBER) {
+                                Value::RangeType range(v1.toDouble(), v2.toDouble(), v3.toDouble());
+                                return Value(range);
+                        }
+                }
 		return Value();
 	}
 	if (this->type == "V") {
@@ -127,13 +161,14 @@ Value Expression::evaluate(const Context *context) const
 		return Value();
 	}
 	if (this->type == "F") {
-		Value::VectorType argvalues;
-		std::transform(this->children.begin(), this->children.end(), 
-									 std::back_inserter(argvalues), 
-									 boost::bind(&Expression::evaluate, _1, context));
-		// for (size_t i=0; i < this->children.size(); i++)
-		// 	argvalues.push_back(this->children[i]->evaluate(context));
-		return context->evaluate_function(this->call_funcname, this->call_argnames, argvalues);
+		FuncRecursionGuard g(*this);
+		if (g.recursion_detected()) { 
+			PRINTB("ERROR: Recursion detected calling function '%s'", this->call_funcname);
+			return Value();
+		}
+
+		EvalContext c(context, this->call_arguments);
+		return context->evaluate_function(this->call_funcname, &c);
 	}
 	abort();
 }
@@ -144,23 +179,31 @@ std::string Expression::toString() const
 
 	if (this->type == "*" || this->type == "/" || this->type == "%" || this->type == "+" ||
 			this->type == "-" || this->type == "<" || this->type == "<=" || this->type == "==" || 
-			this->type == "!=" || this->type == ">=" || this->type == ">") {
+			this->type == "!=" || this->type == ">=" || this->type == ">" ||
+			this->type == "&&" || this->type == "||") {
 		stream << "(" << *this->children[0] << " " << this->type << " " << *this->children[1] << ")";
 	}
 	else if (this->type == "?:") {
-		stream << "(" << *this->children[0] << " ? " << this->type << " : " << *this->children[1] << ")";
+		stream << "(" << *this->children[0] << " ? " << *this->children[1] << " : " << *this->children[2] << ")";
 	}
 	else if (this->type == "[]") {
-		stream << "(" << *this->children[0] << "[" << *this->children[1] << "])";
+		stream << *this->children[0] << "[" << *this->children[1] << "]";
 	}
 	else if (this->type == "I") {
-		stream << "(-" << *this->children[0] << ")";
+		stream << "-" << *this->children[0];
+	}
+	else if (this->type == "!") {
+		stream << "!" << *this->children[0];
 	}
 	else if (this->type == "C") {
 		stream << this->const_value;
 	}
 	else if (this->type == "R") {
-		stream << "[" << *this->children[0] << " : " << *this->children[1] << " : " << this->children[2] << "]";
+		stream << "[" << *this->children[0] << " : " << *this->children[1];
+		if (this->children.size() > 2) {
+			stream << " : " << *this->children[2];
+		}
+		stream << "]";
 	}
 	else if (this->type == "V") {
 		stream << "[";
@@ -174,14 +217,15 @@ std::string Expression::toString() const
 		stream << this->var_name;
 	}
 	else if (this->type == "N") {
-		stream << "(" << *this->children[0] << "." << this->var_name << ")";
+		stream << *this->children[0] << "." << this->var_name;
 	}
 	else if (this->type == "F") {
 		stream << this->call_funcname << "(";
-		for (size_t i=0; i < this->children.size(); i++) {
+		for (size_t i=0; i < this->call_arguments.size(); i++) {
+			const Assignment &arg = this->call_arguments[i];
 			if (i > 0) stream << ", ";
-			if (!this->call_argnames[i].empty()) stream << this->call_argnames[i]  << " = ";
-			stream << *this->children[i];
+			if (!arg.first.empty()) stream << arg.first  << " = ";
+			stream << *arg.second;
 		}
 		stream << ")";
 	}
