@@ -14,6 +14,7 @@
 */
 BoundingBox operator*(const Transform3d &m, const BoundingBox &box)
 {
+	if (box.isEmpty()) return box;
 	BoundingBox newbox;
 	Vector3d boxvec[2] = { box.min(), box.max() };
 	for (int k=0;k<2;k++) {
@@ -46,3 +47,17 @@ bool matrix_contains_nan( const Transform3d &m )
 	return false;
 }
 
+double getBoundingRadius(BoundingBox bbox)
+{
+	// FIXME: For eigen3, we can use bbox.diagonal().norm()/2;
+  double radius = (bbox.max() - bbox.min()).norm() / 2;
+  return radius; // 0;
+}
+
+Vector3d getBoundingCenter(BoundingBox bbox)
+{
+  // FIXME: For eigen3, we can use bbox.center();
+  Vector3d center = (bbox.min() + bbox.max()) / 2;
+  return center; // Vector3d(0,0,0);
+}
+
