@@ -4,10 +4,13 @@
 #include <Eigen/Core>
 #include <Eigen/Geometry>
 #include <Eigen/Dense>
+#include<Eigen/StdVector>
 
+EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION(Eigen::Vector2d)
 using Eigen::Vector2d;
+EIGEN_DEFINE_STL_VECTOR_SPECIALIZATION(Eigen::Vector3d)
 using Eigen::Vector3d;
-using Eigen::Vector3f;
+
 typedef Eigen::AlignedBox<double, 3> BoundingBox;
 using Eigen::Matrix3f;
 using Eigen::Matrix3d;
@@ -22,6 +25,9 @@ bool matrix_contains_infinity( const Transform3d &m );
 bool matrix_contains_nan( const Transform3d &m );
 
 BoundingBox operator*(const Transform3d &m, const BoundingBox &box);
+Vector3d getBoundingCenter(BoundingBox bbox);
+double getBoundingRadius(BoundingBox bbox);
+
 
 class Color4f : public Eigen::Vector4f
 {
