@@ -27,8 +27,7 @@
 #ifndef OPENSCAD_H
 #define OPENSCAD_H
 
-extern class Module *parse(const char *text, const char *path, int debug);
-extern int get_fragments_from_r(double r, double fn, double fs, double fa);
+extern class FileModule *parse(const char *text, const char *path, int debug);
 
 #include <string>
 extern std::string commandline_commands;
