@@ -13,4 +13,10 @@
  #endif
 #endif
 
+#include <string>
+
+std::string glew_dump();
+std::string glew_extensions_dump();
+bool report_glerror(const char * function);
+
 #endif
